### Functional testing

The purpose of functional testing is to the function of component integrations. These tests should be as comprehensive as time allows. Below are some basic examples to get you started.

Note: The `AdminTestCase` uses the `Bazookas\AdminBundle\Tests\DataFixtures\AdminTestDataFixtures` as a default fixture. This class contains several users with different roles to assist with testing admin functionality.

#### Basic success/fail test

This test ensure that all of the pages are accessible by an admin user or inaccessible when not logged in. In `<project directory>/tests/FunctionalTests` create a new class `AdminFunctionTests`. This class should extends the `AdminTestCase` as this provides some basic convenience methods for testing cms related functionalities. An example below:

```php
<?php

namespace tests\FunctionalTests;

use Bazookas\AdminBundle\Controller\ListBaseController;
use Bazookas\AdminBundle\Tests\TestCase\AdminTestCase;

class AdminFunctionTests extends AdminTestCase
{

  /**
   * Test that pages load successfully when logged in as admin
   * @return array
   */
    public function getUrls() {
      $routes = [
        'website_manufacturer_admin',
        'website_model_admin',
        'website_vendor_incentive_admin',
        'website_sales_order_admin',
        'website_customer_type_admin'
      ];
    
      $urls = [];
      foreach($routes as $route) {
        $urls[] = $this->getUrl($route, [
          '_locale' => $this->defaultLocale,
          '_action' => ListBaseController::ACTION_LIST
        ]);
      }
    
      $parentUrls = parent::getUrls();
      return array_merge($urls, $parentUrls);
    }

}
```

Overwrite the `getUrls` function so that it returns a list of relative paths. Each of these paths will be tested to ensure that the page loads when logged in with an admin super user and that they fail when not logged in. 

You can also test for specific routes as follows:

```php
public function testRoleAdminUser() {
    $adminUser = $this->referenceRepository->getReference('user-admin1');

    $this->testPageIsSuccessful($adminUser, [
      $this->getUrl('cms_user_admin', [
        '_locale' => $this->defaultLocale,
        '_action' => ListBaseController::ACTION_LIST
      ])
    ]);
  }

```

The `testPageIsSuccessful` function accepts an optional admin user and an optional list of urls to check. Using this you can quickly test access to different areas of the admin backend based on different user roles. These same parameters can also be passed to `testPageFails`.

#### Interacting with the database

The functional tests use a separate sqlite database to carry out the tests. 

##### Writing fixtures

In order to be able to load a fixture you must first create a fixture class. An example implementation can be found in [AdminTestDataFixtures](Tests\DataFixtures\AdminTestDataFixtures). It is important that your fixture class extend the `AbstractFixture` class from Doctrine. This class allows you to create reference to items you've loaded using your fixtures. In this way you don't need to first retrieve an entity from the database before carrying out an assertion. For example:


```php
<?php

namespace Bazookas\AdminBundle\Tests\DataFixtures;

use Bazookas\AdminBundle\DataFixtures\ORM\AdminFixtures;
use Bazookas\AdminBundle\Entity\CMSUser;
use Bazookas\AdminBundle\Entity\CMSUserGroup;
use Doctrine\Common\Persistence\ObjectManager;

class AdminTestDataFixtures extends AdminFixtures
{
  public function load(ObjectManager $manager)
    {
      $basicGroup = new CMSUserGroup();
      $basicGroup->setName('Basic user');
      $basicGroup->setRoles(array('ROLE_CMS_USER'));
      $manager->persist($basicGroup);
  
      $user = new CMSUser();
      $password = base64_encode(random_bytes(6));
  
      $user
        ->setUsername('basicUser1')
        ->setPlainPassword($password)
        ->setName('basicUser1')
        ->setSurname('Bazookas')
        ->setLocale('nl')
        ->setEmail('test1@bazookas.be')
        ->setEnabled(TRUE)
        ->addGroup($basicGroup)
      ;
      $this->setReference('user-basic1', $user);
      $manager->persist($user);
  }
}
```

The function call `$this->setReference('user-basic1', $user);` allows you to later retrieve the user from the reference repository by calling

```php
$adminUser = $this->referenceRepository->getReference('user-basic1');
```

Note that this function is only available if your functional test class extends the `AdminTestCase`.
