<?php

namespace Bazookas\ExportBundle\Services\Data;

use Bazookas\ExportBundle\Services\Interfaces\DataSelectorInterface;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\QueryBuilder;
use Symfony\Component\Translation\TranslatorInterface;

class DefaultDataSelectorService implements DataSelectorInterface {

  protected $entityManager;

  protected $translator;

  public function __construct(EntityManagerInterface $entityManager, TranslatorInterface $translator) {
    $this->entityManager = $entityManager;
    $this->translator = $translator;
  }

  /**
   * @param array $exportConfig
   * @return array
   */
  public function getData(array $exportConfig) {
    $exportSheets = $exportConfig['sheets'];
    $result = [];
    foreach ($exportSheets as $sheet) {
      $result[] = $this->processSheet($sheet);
    }

    return $result;
  }

  protected function processSheet(array $config) {
    $qb = $this->getQueryBuilder($config);

    // get scalar result so we don't need to loop arrays recursively
    $data = $qb->getQuery()->getArrayResult();

    //Create the data array
    $fields = $config['fields'];

    //Process the data
    $dataArray = $this->createDataArray($fields, $data);

    //Now translate the headers fields
    $headers = $this->translateHeaders(array_values($fields));

    //Set the output
    $sheetData = [];
    $sheetData['worksheet'] = $config['sheetName'];
    $sheetData['headers'] = $headers;
    $sheetData['data'] = $dataArray;

    return $sheetData;
  }

  /**
   * @param array $config
   * @return \Doctrine\ORM\QueryBuilder|mixed
   */
  protected function getQueryBuilder(array &$config) {
    if (!empty($config['queryBuilder'])) {
      return $config['queryBuilder'];
    } else {
      $selectArray = [];
      $joinArray = [];
      $this->convertFields($config['fields'], $joinArray, $selectArray);
      $selectString = implode(', ', $selectArray);

      $qb = $this->entityManager->createQueryBuilder();
      $qb
        ->select($selectString)
        ->from($config['entity'], 'e')
      ;

      foreach ($joinArray as $join => $alias) {
        $qb->join($join, $alias);
      }

      return $qb;
    }
  }

  /**
   * @param array $original
   * @param array $joinArray
   * @param array $selectArray
   */
  protected function convertFields(array &$original, array &$joinArray, array &$selectArray) {
    // make sure the root entity is selected
    $tempSelectArray = [
      'e' => ['id']
    ];

    foreach ($original as $key => &$settings) {
      // while we are looping, we can translate the field header
      if (is_string($settings)) {
        $settings = $this->translator->trans($settings, [], 'admin');
      }

      // create an exploded array to create all joins and selects
      $exploded = explode('.', $key);

      // the last element is the actual field
      $selector = array_pop($exploded);

      // we start the selects from `e`, this is the default selected entity
      $joinedEntity = 'e';

      if (count($exploded) > 0) {
        foreach ($exploded as $relation) {
          // create a join and alias
          // by using the join as key we prevent duplicates
          $joinArray[$joinedEntity . '.' . $relation] = $joinedEntity . '_' . $relation;

          // remember the alias for select statements
          $joinedEntity .= '_' . $relation;

          // the following line is to prevent entities from being skipped in the select
          // id is the minimum required field for selecting entities
          // TODO make this work for identifiers not called `id`
          if (empty($tempSelectArray[$joinedEntity]) || !in_array('id', $tempSelectArray[$joinedEntity])) {
            $tempSelectArray[$joinedEntity][] = 'id';
          }
        }
      }

      // when "settings" is false, it means we don't need to select the field
      // just make sure everything is joined
      if ($settings === false) {
        unset($original[$key]);
      }
      else {
        $tempSelectArray[$joinedEntity][] = $selector;
      }
    }

    foreach ($tempSelectArray as $alias => $fields) {
      // make sure there are no duplicates
      $fields = array_unique($fields);

      // make sure the required id is added to the select statement
      // TODO make this work for identifiers not called `id`
      if (array_search('id', $fields) === false) {
        array_unshift($fields, 'id');
      }
      $selectArray[] = 'partial ' . $alias . '.{' . implode(',',$fields) . '}';
    }
  }

  /**
   * @param array $fields
   * @param array $data
   * @return array
   */
  protected function createDataArray(array &$fields, array $data) {
    $dataArray = [];
    foreach ($data as $row) {
      $rowData = [];
      foreach ($fields as $key => $value){
        $fieldKeys = explode('.', $key);

        if (count($fieldKeys) > 0) {
          $temp = null;
          foreach($fieldKeys as $index => $fieldKey) {
            if ($index == 0) {
              $temp = $row[$fieldKey];
            } else {
              if (isset($temp[$fieldKey])) {
                $temp = $temp[$fieldKey];
              } else {
                $temp = null;
              }
            }
          }

          if ($temp instanceof \DateTime) {
            $temp->setTimezone(new \DateTimeZone('Europe/Brussels'));
            $temp = $temp->format('Y-m-d H:i');
          }

          if (is_bool($temp)) {
            $temp = (int) $temp;
          }

          $rowData[] = $temp;
        } else {
          $rowData[] = $row[$fieldKeys[0]];
        }
      }
      $dataArray[] = $rowData;
    }

    return $dataArray;
  }

  /**
   * @param array $headers
   * @return array
   */
  protected function translateHeaders(array $headers) {
    foreach($headers as $key => $header) {
      $headers[$key] = $this->translator->trans($header, [], 'admin');
    }

    return $headers;
  }
}
